/* Copyright (c) 2012, David Melik and Ben Collver
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.
3. All advertising materials mentioning features or use of this software must
   display the following acknowledgement:
   This product includes software developed by David Melik and Ben Collver.
4. The names of its contributors may not be used to endorse or promote products
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY DAVID MELIK AND BEN COLLVER ''AS IS'' AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

//graphics programmed by David, with some SDL code from Ben

//Ben's comments:
// g++ $(sdl-config --cflags) demo.C $(sdl-config --libs) -lSDL_gfx

/* for double-buffering information, see:
http://sdldoc.csn.ul.ie/sdlflip.php
http://www.linuxdevcenter.com/pub/a/linux/2003/08/07/sdl_anim.html 
*/

/* for graphics primitives, see:
http://www.ferzkopp.net/~aschiffler/Software/SDL_gfx-2.0/
*/

#define gcc  //GNU Compiler Collection--originally compiled on GNU/Linux
//#define tc //Turbo C/C++--not finished for this yet
//#define wc //Watcom C/C++ (for DOS)
#ifdef gcc
#include <SDL/SDL.h>
#include <SDL/SDL_gfxPrimitives.h>
#endif
#ifdef tc
#include <graphics.h> //initgraph()
#endif
#ifdef tc
#include <conio.h>    // clrscr(), getch()
#endif
#ifdef wc
#include <conio.h>    //getch()
#include <graph.h>    //_getactivepage(), _getvisualpage(), _setactivepage(), _setvisualpage()
#include <i86.h>      //delay()
#endif
#include <math.h>     //ceil(), cos(), floor(), sin(), sqrt()
#include <stdio.h>    //printf()
//#include <stdlib.h>   //abs()

#ifdef gcc
void clrscr(SDL_Surface *screen,SDL_Color colour);
void delay(int n);
void getch(void);
void put_pixel(SDL_Surface *screen,int x,int y,SDL_Color colour);
void line(SDL_Surface *screen,int x0,int y0,int x1,int y1,SDL_Color colour);
void draw_square(SDL_Surface *screen,int square[][4],SDL_Color colour);
void draw_cube(SDL_Surface *screen,int cube[][8],int z,SDL_Color colour);
#endif
#ifdef wc
void put_pixel(int x,int y,int colour);
void line(int x0,int y0,int x1,int y1,int colour);
//void circle(int radius,int h,int k,int color);
void draw_square(int square[][4],int colour);
void draw_cube(int cube[][8],int z,int colour);
#endif
//void translate_shape(int shape[][sizeof(int / 2)], int colour);
//Watcom
//void wait_retrace();
#ifdef wc
void clrscr();
#endif
//int sign(int);
//void matrix_mul(int v, int [][sizeof(m/v)]m, int [sizeof(n/v)][]n);
//int oprod(int v1[3], int v2[3]);
//void xprod(int v1[3], int v2[3], int *vx[3]);

int main(void)
{
  int i;
    //define polytopes
  int square_100[2][4]={{50,  50, -50, -50},
                        {50, -50, -50,  50}};
  int square_a[2][4]={{50,  50, -50, -50},
                      {50, -50, -50,  50}};
    //define polyhedra
  int cube_100[3][8]={{-50, -50,  50,  50, -50, -50, 50,  50},
                      {-50,  50,  50, -50, -50,  50, 50, -50},
                      {-50, -50, -50, -50,  50,  50, 50,  50}};
  int cube_a[3][8]={{-50, -50,  50,  50, -50, -50, 50,  50},
                    {-50,  50,  50, -50, -50,  50, 50, -50},
                    {-50, -50, -50, -50,  50,  50, 50,  50}};
  float x,y,z;
    //define 3D angles and rotation matrices
    //alpha, beta, lambda
  //float a=0;
  float a=0.785398; //pi/4;
  //float b=0;
  float b=0.785398;
  float l=0.785398;
  float ml[2][2]={{ cos(l), sin(l)},
                  {-sin(l), cos(l)}};
  float mR3a[3][3]={{1, 0,      0},
                    {0, cos(a), -sin(a)},
                    {0, sin(a), cos(a)}};
  float mR3b[3][3]={{cos(b),  0, sin(b)},
                    {      0, 1,      0},
                    {-sin(b), 0, cos(b)}};
  float mR3l[3][3]={{cos(l), -sin(l), 0},
                    {sin(l),  cos(l), 0},
                    {     0,       0, 1}};
  float mR3r[3][3]={{ cos(b)*cos(l)+sin(a)*sin(b)*sin(l), -cos(a)*sin(l)+cos(l)*sin(a)*sin(b), cos(a)*sin(b)},
                    {                      cos(a)*sin(l),                       cos(a)*cos(l),       -sin(a)},
                    {-cos(l)*sin(b)+cos(b)*sin(a)*sin(l),  sin(b)*sin(l)+cos(b)*cos(l)*sin(a), cos(a)*cos(b)}};
  #ifdef gcc
  SDL_Surface *screen;
  SDL_Color black={0x00,0x00,0x00,0x00};
  SDL_Color green={0x00,0xFF,0x00,0x00};
  #endif

  for(i=0;i<=3;i++)
  {
    square_a[0][i]+=320;
    square_a[1][i]+=240;
  }
  /*//clearscreen, set video mode 320*240*16, error if impossible
  clrscr();
  if (_setvideomode(_MRES16COLOR) == 0)
  {
    printf("%s", "Cannot set 320x200x16 video mode.\r\n");
    return 1;
  } */
    //clearscreen, set video mode to 640x480x16, error if impossible
  #ifdef gcc
  if (SDL_Init(SDL_INIT_VIDEO)<0)
  {
    fprintf(stderr,"Cannot start SDL: %s\n", SDL_GetError());
    return 1;
  }
  atexit(SDL_Quit);
  screen=SDL_SetVideoMode(640,480,8,SDL_SWSURFACE);
  if (screen==NULL)
  {
    fprintf(stderr,"Cannot set 640x480x256 video mode: %s\n",
    SDL_GetError());
    return 1;
  }
  clrscr(screen, black);
  //SDL_UpdateRect(screen,0,0,0,0);
  getch();
  put_pixel(screen,320,240,green);
  SDL_UpdateRect(screen,0,0,0,0);
  getch();
  draw_square(screen,square_a,green);
  SDL_UpdateRect(screen,0,0,0,0);
  getch();
  #endif
  //initgraph(VGA, 2, ""); //almost right in Turbo C
  #ifdef wc
  clrscr();
  if(_setvideomode(_VRES16COLOR)==0)
  {
    printf("%s","Cannot set 640x480x16 video mode.\r\n");
    return 1;
  }
  put_pixel(320,240,2);
  getch();
  draw_square(square_a,2);
  getch();
  #endif

    //rotation
    //pi/24=0.130899
    //pi/30=0.104719
    //pi/60=0.052360 optimal at 640*480
  for(l=0;l<=6.283185;l+=0.104719)
  {
    //square rotation matrices, defining trigonometric functions
    ml[0][0]=cos(l);
    ml[0][1]=sin(l);
    ml[1][0]=-ml[0][1];
    ml[1][1]=ml[0][0];
    for(i=0; i<=3; i++)
    {
      //printf("%s%d%s%d%s%d\n", "old quad: ", i+1, "x,y: ", square_100[0][i], " ", square_100[1][i], "\n");
      x=ml[0][0]*square_100[0][i]+ml[0][1]*square_100[1][i];
      y=ml[1][0]*square_100[0][i]+ml[1][1]*square_100[1][i];
      square_a[0][i]=(int)x;
      square_a[1][i]=(int)y;
      //printf("%s%d%s%d%s%d\n", "new quad: ", i+1, "x,y: ", square_100[0][i], " ", square_100[1][i], "\n");
    }
    for(i=0;i<=3;i++)
    {
      square_a[0][i]+=320;
      square_a[1][i]+=240;
    }
    #ifdef gcc
    delay(10);
    clrscr(screen,black);
    draw_square(screen,square_a,green);
    SDL_UpdateRect(screen,0,0,0,0);
    #endif
    #ifdef wc
    //delay(1);
    clrscr();
    draw_square(square_a,2);
    #endif
  }
  getch();
  #ifdef gcc
  clrscr(screen,black);
  #endif
  #ifdef wc
  clrscr();
  #endif

/* uncomment and change declarations above for strict C
    //cube rotation matrices
  mR3a[0][0]=1;
  mR3a[0][1]=0;
  mR3a[0][2]=0;
  mR3a[1][0]=0;
  mR3a[1][1]=cos(b);
  mR3a[1][2]=sin(b);
  mR3a[2][0]=0;
  mR3a[2][1]=-mR3a[1][2];
  mR3a[2][2]=mR3a[1][1];
  mR3b[0][0]=cos(a);
  mR3b[0][1]=0;
  mR3b[0][2]=-sin(a);
  mR3b[1][0]=0;
  mR3b[1][1]=1;
  mR3b[1][2]=0;
  mR3b[2][0]=-mR3b[0][2];
  mR3b[2][1]=0;
  mR3b[2][2]=mR3b[0][0];
  mR3l[0][0]=cos(a);
  mR3l[0][1]=sin(a);
  mR3l[0][2]=0;
  mR3l[1][0]=-mR3l[0][1];
  mR3l[1][1]=mR3l[0][0];
  mR3l[1][2]=0;
  mR3l[2][0]=0;
  mR3l[2][1]=0;
  mR3l[2][2]=1;*/
  #ifdef gcc
  draw_cube(screen,cube_a,256,green);
  SDL_UpdateRect(screen,0,0,0,0);
  getch();
  #endif
  #ifdef wc
  draw_cube(cube_a,256,2);
  getch();
  #endif
  for(l=0;l<=6.283185;l+=0.104719)
  {
    mR3r[0][0]=cos(b)*cos(l)+sin(a)*sin(b)*sin(l);
    mR3r[0][1]=-cos(a)*sin(l)+cos(l)*sin(a)*sin(b);
    mR3r[0][2]=cos(a)*sin(b);
    mR3r[1][0]=cos(a)*sin(l);
    mR3r[1][1]=cos(a)*cos(l);
    mR3r[1][2]=-sin(a);
    mR3r[2][0]=-cos(l)*sin(b)+cos(b)*sin(a)*sin(l);
    mR3r[2][1]=sin(b)*sin(l)+cos(b)*cos(l)*sin(a);
    mR3r[2][2]=cos(a)*cos(b);
    for(i=0; i<=7; i++)
    {
      x=mR3r[0][0]*cube_100[0][i]+mR3r[0][1]*cube_100[1][i]+mR3r[0][2]*cube_100[2][i];
      y=mR3r[1][0]*cube_100[0][i]+mR3r[1][1]*cube_100[1][i]+mR3r[1][2]*cube_100[2][i];
      z=mR3r[2][0]*cube_100[0][i]+mR3r[2][1]*cube_100[1][i]+mR3r[2][2]*cube_100[2][i];
      cube_a[0][i]=(int)x;
      cube_a[1][i]=(int)y;
      cube_a[2][i]=(int)z;
    }
    #ifdef gcc
    delay(10);
    clrscr(screen,black);
    draw_cube(screen,cube_a,256,green);
    SDL_UpdateRect(screen,0,0,0,0);
    #endif
    #ifdef wc
    //delay(1);
    clrscr();
    draw_cube(cube_a,256,2);
    #endif
  }

    //wait; set video mode to text; exit
  getch();
  #ifdef wc
  _setvideomode(_TEXTC80);
  #endif
  printf("%s","Programmed by David Chmelik and Ben Collver. (http://www.cwu.edu/~melikd/, http://www.terrorpin.net/\n");
  return 0;
}

#ifdef gcc
void clrscr(SDL_Surface *screen, SDL_Color colour)
{
  SDL_FillRect(screen,NULL,SDL_MapRGB(screen->format, colour.r, colour.g, colour.b));
  SDL_UpdateRect(screen, 0, 0, 0, 0);
}

void delay(int n)
{
  SDL_Delay(n);
}

void getch(void)
{
  SDL_Event event;
  int done=0;
  while(!done){
    if(SDL_WaitEvent(&event)<0)
    {
      fprintf(stderr, "SDL_WaitEvent() error: %s\n",
      SDL_GetError());
      done=1;
      continue;
    }
    if (event.type == SDL_KEYUP) done=1;
  }
}

void put_pixel(SDL_Surface *screen, int x, int y, SDL_Color colour)
{
  pixelRGBA(screen,x,y,colour.r,colour.g,colour.b,0xFF);
}

void line(SDL_Surface *screen, int x0, int y0, int x1, int y1, SDL_Color colour)
{
  lineRGBA(screen,x0,y0,x1,y1,colour.r,colour.g,colour.b,0xFF);
}

void draw_square(SDL_Surface *screen,int square[][4],SDL_Color colour)
{
  line(screen,square[0][0],square[1][0],square[0][1],square[1][1], colour);
  line(screen,square[0][1],square[1][1],square[0][2],square[1][2], colour);
  line(screen,square[0][2],square[1][2],square[0][3],square[1][3], colour);
  line(screen,square[0][3],square[1][3],square[0][0],square[1][0], colour);
}

void draw_cube(SDL_Surface *screen, int cube[][8], int z, SDL_Color colour)
{
    line(screen,z*cube[0][0]/(z+cube[2][0])+320,z*cube[1][0]/(z+cube[2][0])+240,z*cube[0][1]/(z+cube[2][1])+320,z*cube[1][1]/(z+cube[2][1])+240, colour);
    line(screen,z*cube[0][0]/(z+cube[2][0])+320,z*cube[1][0]/(z+cube[2][0])+240,z*cube[0][3]/(z+cube[2][3])+320,z*cube[1][3]/(z+cube[2][3])+240, colour);
    line(screen,z*cube[0][0]/(z+cube[2][0])+320,z*cube[1][0]/(z+cube[2][0])+240,z*cube[0][4]/(z+cube[2][4])+320,z*cube[1][4]/(z+cube[2][4])+240, colour);
    line(screen,z*cube[0][1]/(z+cube[2][1])+320,z*cube[1][1]/(z+cube[2][1])+240,z*cube[0][2]/(z+cube[2][2])+320,z*cube[1][2]/(z+cube[2][2])+240, colour);
    line(screen,z*cube[0][1]/(z+cube[2][1])+320,z*cube[1][1]/(z+cube[2][1])+240,z*cube[0][5]/(z+cube[2][5])+320,z*cube[1][5]/(z+cube[2][5])+240, colour);
    line(screen,z*cube[0][2]/(z+cube[2][2])+320,z*cube[1][2]/(z+cube[2][2])+240,z*cube[0][6]/(z+cube[2][6])+320,z*cube[1][6]/(z+cube[2][6])+240, colour);
    line(screen,z*cube[0][3]/(z+cube[2][3])+320,z*cube[1][3]/(z+cube[2][3])+240,z*cube[0][2]/(z+cube[2][2])+320,z*cube[1][2]/(z+cube[2][2])+240, colour);
    line(screen,z*cube[0][3]/(z+cube[2][3])+320,z*cube[1][3]/(z+cube[2][3])+240,z*cube[0][7]/(z+cube[2][7])+320,z*cube[1][7]/(z+cube[2][7])+240, colour);
    line(screen,z*cube[0][4]/(z+cube[2][4])+320,z*cube[1][4]/(z+cube[2][4])+240,z*cube[0][7]/(z+cube[2][7])+320,z*cube[1][7]/(z+cube[2][7])+240, colour);
    line(screen,z*cube[0][5]/(z+cube[2][5])+320,z*cube[1][5]/(z+cube[2][5])+240,z*cube[0][6]/(z+cube[2][6])+320,z*cube[1][6]/(z+cube[2][6])+240, colour);
    line(screen,z*cube[0][5]/(z+cube[2][5])+320,z*cube[1][5]/(z+cube[2][5])+240,z*cube[0][4]/(z+cube[2][4])+320,z*cube[1][4]/(z+cube[2][4])+240, colour);
    line(screen,z*cube[0][7]/(z+cube[2][7])+320,z*cube[1][7]/(z+cube[2][7])+240,z*cube[0][6]/(z+cube[2][6])+320,z*cube[1][6]/(z+cube[2][6])+240, colour);
}
#endif

#ifdef wc
void clrscr()
{
  _clearscreen(_GCLEARSCREEN);
}

void put_pixel(int x, int y, int colour)
{
  _moveto(x, y);
  _setcolor(colour);
  _setpixel(x, y);
}

void line(int x0, int y0, int x1, int y1, int colour)
{
  _moveto(x0,y0);
  _setcolor(colour);
  _lineto(x1,y1);
}

void draw_square(int square[2][4], int colour)
{
    line(square[0][0], square[1][0], square[0][1], square[1][1], colour);
    line(square[0][1], square[1][1], square[0][2], square[1][2], colour);
    line(square[0][2], square[1][2], square[0][3], square[1][3], colour);
    line(square[0][3], square[1][3], square[0][0], square[1][0], colour);
}


void draw_cube(int cube[][8], int z, int colour)
{
    line(z*cube[0][0]/(z+cube[2][0])+320,z*cube[1][0]/(z+cube[2][0])+240,z*cube[0][1]/(z+cube[2][1])+320,z*cube[1][1]/(z+cube[2][1])+240, colour);
    line(z*cube[0][0]/(z+cube[2][0])+320,z*cube[1][0]/(z+cube[2][0])+240,z*cube[0][3]/(z+cube[2][3])+320,z*cube[1][3]/(z+cube[2][3])+240, colour);
    line(z*cube[0][0]/(z+cube[2][0])+320,z*cube[1][0]/(z+cube[2][0])+240,z*cube[0][4]/(z+cube[2][4])+320,z*cube[1][4]/(z+cube[2][4])+240, colour);
    line(z*cube[0][1]/(z+cube[2][1])+320,z*cube[1][1]/(z+cube[2][1])+240,z*cube[0][2]/(z+cube[2][2])+320,z*cube[1][2]/(z+cube[2][2])+240, colour);
    line(z*cube[0][1]/(z+cube[2][1])+320,z*cube[1][1]/(z+cube[2][1])+240,z*cube[0][5]/(z+cube[2][5])+320,z*cube[1][5]/(z+cube[2][5])+240, colour);
    line(z*cube[0][2]/(z+cube[2][2])+320,z*cube[1][2]/(z+cube[2][2])+240,z*cube[0][6]/(z+cube[2][6])+320,z*cube[1][6]/(z+cube[2][6])+240, colour);
    line(z*cube[0][3]/(z+cube[2][3])+320,z*cube[1][3]/(z+cube[2][3])+240,z*cube[0][2]/(z+cube[2][2])+320,z*cube[1][2]/(z+cube[2][2])+240, colour);
    line(z*cube[0][3]/(z+cube[2][3])+320,z*cube[1][3]/(z+cube[2][3])+240,z*cube[0][7]/(z+cube[2][7])+320,z*cube[1][7]/(z+cube[2][7])+240, colour);
    line(z*cube[0][4]/(z+cube[2][4])+320,z*cube[1][4]/(z+cube[2][4])+240,z*cube[0][7]/(z+cube[2][7])+320,z*cube[1][7]/(z+cube[2][7])+240, colour);
    line(z*cube[0][5]/(z+cube[2][5])+320,z*cube[1][5]/(z+cube[2][5])+240,z*cube[0][6]/(z+cube[2][6])+320,z*cube[1][6]/(z+cube[2][6])+240, colour);
    line(z*cube[0][5]/(z+cube[2][5])+320,z*cube[1][5]/(z+cube[2][5])+240,z*cube[0][4]/(z+cube[2][4])+320,z*cube[1][4]/(z+cube[2][4])+240, colour);
    line(z*cube[0][7]/(z+cube[2][7])+320,z*cube[1][7]/(z+cube[2][7])+240,z*cube[0][6]/(z+cube[2][6])+320,z*cube[1][6]/(z+cube[2][6])+240, colour);
}
#endif

    // Bresenham or fast line
/* void line(int x0, int y0, int x1, int y1, int colour)
{
  int dx, dy, flength, mid, nlength, xrun, yrun, xrise, yrise;
  dx=x1-x0;
  dy=y1-y0;
  flength=abs(dx);
  mid=floor(flength/2);
  nlength=abs(dy);
  xrun=sign(dx);
  yrun=sign(dy);
  xrise=xrun;
  yrise=0;
  if(nlength>=flength)
  {
    xrise=0;
    yrise=sign(dy);
    flength=abs(dy);
    nlength=abs(dx);
  }
  for(int i=0;i<nlength;i++)
  {
    put_pixel(x0,y0,colour);
    mid+=nlength;
    if (mid>=flength)
    {
      mid-=flength;
      x0+=xrun;
      y0+=yrun;
    }
    else
    {
      x0+=xrise;
      y0+=yrise;
    }
  }
} */

    //_ellipse(_GBORDER, 10, 10, 30, 30);
/* circle WIP... '^' needs to be changed to pow()
void circle(int radius, int h, int k, int color)
{
  int x, y;
  for(x = (radius - h) / 1.4142; x >= (radius + h) / 1.4142; x++)
  {
    put_pixel(x, floor(k + sqrt(radius^2 + x^2)), color);
    put_pixel(floor(h + sqrt(radius^2 + y^2)), y, color);
    put_pixel(x, 0 - floor(k + sqrt(radius^2 + x^2)), color);
    put_pixel(0 - floor(h + sqrt(radius^2 + y^2)), y, color);
    put_pixel(0 - x, 0 - floor(k + sqrt(radius^2 + x^2)), color);
    put_pixel(0 - floor(h + sqrt(radius^2 + y^2)), 0 - y, color);
    put_pixel(0 - x, floor(k + sqrt(radius^2 + x^2)), color);
    put_pixel(0 - floor(h + sqrt(radius^2 + y^2)), y, color);
  }
} */

/* early notes
void draw_cube(int cube[3][8], int colour)
{
  //cull, draw side lines

  //check if face Z, Y, or -Y || to screen, and which visible
  if((cube[0][0]==cube[0][6])||(cube[0][0]==cube[2][0])
  {
    if(cos(o1)==90)
    {
    }
  }
  //check if Y is perspectived
  else if((cube[2][3]==cube[2][2])||(cube[0][0]==cube[2][0])
  {
    if(cos(o2)==90)
    {
      if(cos(o3)==90)
      {
      //if not check z else others, otherwise next if
      }
    }
  }
  else if((cube[2][5]==cube[2][4])||(cube[0][0]==cube[2][0])
  {
    //another face also perspectived
    if(cos(o2)==90)
    if(cos(o3)==90)
    if(cos(o4)==90)
      4 drawing cases unless last if optimized
  }
} */

    //matehmatical function to give the numberical sign of a number
/*int sign(int n)
{
    //find sign of n
  if(n>0)
    return 1;
  else if(n<0)
    return -1;
  else return 0;
}*/

  //dot product
/*int oprod(int v1[], int v2[])
{
  return v1[0]*v2[0]+v1[1]*v2[1]+v1[2]*v2[2];
}*/

/*  //cross product
int* xprod(int v1[3], int v2[3], int *vx[3])
{
  return (v1[0]*v2[1]-v1[1]*v2[0], v1[1]*v2[2]-v1[2]*v2[1], v1[2]*v2[3]-v1[3]*v2[2]);
} */

    //non-working slow version
/*void line(int x0, int y0, int x1, int y1, int color)
{
  float x = x0;
  float y = y0;
  int dx = x1 - x0;
  int dy = y1 - y0;
  if((!dx) && (dy < 0))
  {
    for(y = y0; y <= y1; y--)
      put_pixel(x, y, color);
    return;
  }
  else if((!dx) && (!dy))
  {
    put_pixel(x, y, color);
    return;
  }
  else if((!dx) && (dy > 0))
  {
    for(y = y0; y >= y1; y++)
      put_pixel(x, y, color);
    return;
  }
  else if(dx)
  {
    if(sign(dy) < 0)
    {
      float m = dx / dy;
      if(dx < 0)
      {
	for(x = x0; x <= x1; x--)
	{
	  put_pixel(x, floor(y + 0.5), color);
	  y -= m;
	}
	return;
      }
      else if(dx > 0)
      {
	for(x = x0; x >= x1; x++)
	{
	  put_pixel(x, floor(y + 0.5), color);
	  y -= m;
	}
	return;
      }
    }
    else if(!dy)
    {
      if(dx < 0)
      {
	for(x = x0; x <= x1; x--)
	{
	  put_pixel(x, y, color);
	}
	return;
      }
      else if(dx > 0)
      {
	for(x = x0; x >= x1; x++)
	{
	  put_pixel(x, y, color);
	}
	return;
      }
    }
    if(sign(dy) > 0)
    {
      float m = dx / dy;
      if(dx < 0)
      {
	for(x = x0; x <= x1; x--)
	{
	  put_pixel(x, floor(y + 0.5), color);
	  y += m;
	}
	return;
      }
      else if(dx > 0)
      {
	for(x = x0; x >= x1; x++)
	{
	  put_pixel(x, floor(y + 0.5), color);
	  y += m;
	}
	return;
      }
    }
  }
}*/
